﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour {

    public static GameManager gManager;

    GameObject playerPrefab;
    GameObject enemy;
    List<GameObject> enemies = new List<GameObject>();
    List<GameObject> cosas = new List<GameObject>();
    GameObject zone;
    GameObject player;
    GameObject obstaclesZone;
    Text levelText;
    AudioSource _audio;
    bool levelReady = false;
    bool trapActivate = false;

    public RectTransform deadScreen;
    public RectTransform levelScreen;
    public RectTransform pause;
    public GameObject[] obstacles;
    public GameObject[] mesasZone;
    public AudioClip[] audios;

    int vidrios = 2;
    int mesas = 2;
    int tablas = 1;
    int level = 1;

    public enum Directions { UP = 0, DOWN, LEFT, RIGHT }

    Directions dir;

    public void SetTrapActivated(bool status)
    {
        trapActivate = status;
    }

    public bool GetLevelReady()
    {
        return levelReady;
    }

    void Awake()
    {
        if(gManager == null)
        {
            gManager = this;
        }
        zone = GameObject.Find("PlayZone");
        playerPrefab = Resources.Load<GameObject>("Prefabs/Player");
        enemy = Resources.Load<GameObject>("Prefabs/Zombie");
        obstaclesZone = GameObject.Find("ObstaculosSpawn");
        _audio = GetComponent<AudioSource>();
    }

    void Start()
    {
        levelText = levelScreen.GetComponentInChildren<Text>();
        SettingZone();

    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            pause.gameObject.SetActive(true);
        }
        if (!deadScreen.gameObject.activeSelf)
        {
            foreach (GameObject e in enemies)
            {
                e.GetComponent<EnemiesController>().CheckDistance(trapActivate);
            }
        }
    }

    void SettingZone()
    {
        levelScreen.gameObject.SetActive(true);
        levelText.text = "Level: " + level;
        StartCoroutine(LevelSplash());
        if (player == null)
        {
            player = Instantiate(playerPrefab, GameObject.Find("PlayerSpawn").transform);
            player.transform.localPosition = new Vector3(0, 0.5f, 0);
            player.transform.Rotate(new Vector3(0, -135, 0));
        }
        player.GetComponent<PlayerController>().SetAlive();
        for (int i = 0; i < level; i++)
        {
            GameObject en = Instantiate(enemy, zone.transform);
            enemies.Add(en);
            en.transform.localPosition = new Vector3(Random.Range(-2, 5), 0f, Random.Range(-2, 5));
            if (Vector3.Distance(en.transform.position, player.transform.position) < 5)
            {
                while (Vector3.Distance(en.transform.position, player.transform.position) < 5)
                    en.transform.localPosition = new Vector3(Random.Range(0, 5), 0.5f, Random.Range(-2, 5));
            }
        }
        //////////////Objetos///////////////
        for (int i = 0; i < vidrios; i++)
        {
            GameObject v = Instantiate(obstacles[0], obstaclesZone.transform);
            v.transform.localPosition = new Vector3(Random.Range(0, 5), 0.1f, Random.Range(-2, 5));
            cosas.Add(v);
        }
        for (int i = 0; i < mesas; i++)
        {
            GameObject m = Instantiate(Resources.Load<GameObject>("Prefabs/Mesas/1"), mesasZone[Random.Range(0, 3)].transform);
            m.transform.localPosition = new Vector3(Random.Range(0, 5), 0, Random.Range(-2, 5));
            cosas.Add(m);
        }
        
    }

    public void Lose()
    {
        _audio.PlayOneShot(audios[0]);
        deadScreen.gameObject.SetActive(true);
        StartCoroutine(DeadSplash());
    }

    IEnumerator DeadSplash()
    {
        Vector4 fade = new Vector4(0, 0, 0, 0);
        while (fade.w < 1)
        {
            fade += new Vector4(0, 0, 0, 0.01f);
            deadScreen.GetComponent<Image>().color = fade;
            yield return new WaitForEndOfFrame();
        }
    }

    IEnumerator LevelSplash()
    {
        Vector4 fade = new Vector4(0.3f, 0.3f, 0.3f, 1);
        while (fade.w > 0)
        {
            fade -= new Vector4(0, 0, 0, 0.01f);
            levelScreen.GetComponent<Image>().color = fade;
            yield return new WaitForEndOfFrame();
        }
        levelReady = true;
        levelScreen.gameObject.SetActive(false);
    }

    public void LevelComplete()
    {
        level++;
        levelReady = false;
        foreach(GameObject e in enemies)
        {
            Destroy(e);
        }
        enemies.Clear();
        player.transform.localPosition = new Vector3(0, 0.5f, 0);
        if (level % 2 == 0) { vidrios++; }
        if (level % 3 == 0) { mesas++; }
        if (level % 5 == 0) { tablas++; }
        foreach (GameObject c in cosas)
        {
            Destroy(c);
        }
        cosas.Clear();
        trapActivate = false;
        SettingZone();
    }

    public void RestartLevel()
    {
        if (pause.gameObject.activeSelf)
            pause.gameObject.SetActive(false);

        deadScreen.gameObject.SetActive(false);
        levelReady = false;
        foreach (GameObject e in enemies)
        {
            Destroy(e);
        }
        enemies.Clear();
        foreach (GameObject c in cosas)
        {
            Destroy(c);
        }
        cosas.Clear();
        trapActivate = false;
        player.transform.localPosition = new Vector3(0, 0.5f, 0);
        SettingZone();
    }

}
